// mutex.hpp - Mutex acquiring/release

class Mutex {
  friend class Lock;
  HANDLE mutex_;
  DWORD wait_time_;
  DWORD dw_;
  void aquire() 
  {
    dw_ = ::WaitForSingleObject(mutex_, wait_time_);
  }
  void release()
  {
    BOOL IS_OK = ::ReleaseMutex(mutex_);
	if(!IS_OK) {
		::MessageBox(NULL, "Release Mutex failed", "Mutex release", MB_OK );
	}
	return;
  }
public:
  Mutex(const char *mutex_name, DWORD wait_time )
    : wait_time_(wait_time)
  {
    mutex_ = ::CreateMutex(NULL, FALSE, mutex_name);
    if( NULL == mutex_) {
      int ret_error = ::GetLastError();
      std::ostringstream message;
      message << "CreateMutex failed with code"<<std::setw(6)<<ret_error;
      ::MessageBox(NULL, (LPCTSTR) message.str().c_str(), "Mutex:", 
		   MB_OK | MB_ICONERROR );
    }
  }
  ~Mutex() 
  {
    ::CloseHandle(mutex_);
  }
  DWORD wait_return_code() const { return dw_;}
};

class Lock
{
  Mutex& mutex_;
public:
  Lock( Mutex& mutex ) : mutex_(mutex)
  {
    mutex_.aquire();
  }
  ~Lock()
  {
    mutex_.release();
  }
};
